@@ -551,6 +551,12 @@ pub enum Statement {
     Rollback { chain: bool },
     /// CREATE SCHEMA
     CreateSchema { schema_name: ObjectName },
+
+    /// ASSERT <condition> [AS <message>]
+    Assert {
+        condition: Expr,
+        message: Option<Expr>,
+    },
 }
 
 impl fmt::Display for Statement {
@@ -810,6 +816,14 @@ impl fmt::Display for Statement {
                 write!(f, "ROLLBACK{}", if *chain { " AND CHAIN" } else { "" },)
             }
             Statement::CreateSchema { schema_name } => write!(f, "CREATE SCHEMA {}", schema_name),
+            Statement::Assert { condition, message } => {
+                write!(f, "ASSERT {}", condition)?;
+
+                if let Some(m) = message {
+                    write!(f, " AS {}", m)?;
+                }
+                Ok(())
+            }
         }
     }
 }
