@@ -1,5 +1,3 @@
-// Copyright 2018 Grove Enterprises LLC
-//
 // Licensed under the Apache License, Version 2.0 (the "License");
 // you may not use this file except in compliance with the License.
 // You may obtain a copy of the License at
@@ -16,31 +14,32 @@
 //!
 //! Example code:
 //!
-//! This crate provides an ANSI:SQL 2011 lexer and parser that can parsed SQL into an Abstract
-//! Syntax Tree (AST).
+//! This crate provides an ANSI:SQL 2011 lexer and parser that can parse SQL
+//! into an Abstract Syntax Tree (AST).
 //!
 //! ```
-//! use sqlparser::dialect::GenericSqlDialect;
-//! use sqlparser::sqlparser::Parser;
+//! use sqlparser::dialect::GenericDialect;
+//! use sqlparser::parser::Parser;
 //!
-//! let dialect = GenericSqlDialect {}; // or AnsiSqlDialect
+//! let dialect = GenericDialect {}; // or AnsiDialect
 //!
 //! let sql = "SELECT a, b, 123, myfunc(b) \
 //!            FROM table_1 \
 //!            WHERE a > b AND b < 100 \
-//!            ORDER BY a DESC, b NULLS FIRST";
+//!            ORDER BY a DESC, b";
 //!
-//! let ast = Parser::parse_sql(&dialect, sql.to_string()).unwrap();
+//! let ast = Parser::parse_sql(&dialect, sql).unwrap();
 //!
 //! println!("AST: {:?}", ast);
 //! ```
+#![warn(clippy::all)]
 
-#[macro_use]
-extern crate log;
-extern crate chrono;
-extern crate uuid;
-
+pub mod ast;
 pub mod dialect;
-pub mod sqlast;
-pub mod sqlparser;
-pub mod sqltokenizer;
+pub mod parser;
+pub mod tokenizer;
+
+#[doc(hidden)]
+// This is required to make utilities accessible by both the crate-internal
+// unit-tests and by the integration tests <https://stackoverflow.com/a/44541071/1026>
+pub mod test_utils;
